/* File: cell-towers.cpp
 *
 * A program that determines the maximum population that can be covered
 * by purchasing cell towers with a single transmitting frequency.
 */
#include <iostream>
#include <algorithm>
#include "vector.h"
using namespace std;

/* Given a vector, sums up the elements of the vector
 * and returns the result.
 */
int totalOf(Vector<int>& list) {
	int result = 0;
	for (int i = 0; i < list.size(); i++) {
		result += list[i];
	}
	return result;
}

/* Given a nonempty vector, returns a new vector consisting of all
 * the elements after the first.
 */
Vector<int> tailOf(Vector<int>& list) {
	Vector<int> result = list;
	result.removeAt(0);
	return result;
}

/* Given a vector containing the populations of a bunch of cities
 * in a line, returns the maximum number of people to whom cell
 * coverage can be given.
 */
int maximumCoverageFor(Vector<int>& populations) {
	/* Base case 1: If there aren't any cities, then no one can be covered. */
	if (populations.size() == 0) {
		return 0;
	}
	/* Base case 2: If there is just one city, we can cover it completely. */
	else if (populations.size() == 1) {
		return populations[0];
	}
	/* Recursive step. */
	else {
		Vector<int> allButFirst    = tailOf(populations);
		Vector<int> allButFirstTwo = tailOf(allButFirst);
		
		/* There are two options - we either skip the first city (in which case we
		 * can maximize the coverage to the remainder), or we pick the first city,
		 * in which case we must skip the second and then maximize the coverage to
		 * the remainder.  Thus, try out both options and see which one ends up
		 * better.
		 */
		return max(maximumCoverageFor(allButFirst),
		           populations[0] + maximumCoverageFor(allButFirstTwo));
	}
}

int main() {
	/* Populate a sample list of cities. */
	Vector<int> populations;
	populations += 9000, 10000, 9000, 12000, 6000;
	
	cout << "Maximum coverage: " << maximumCoverageFor(populations) << endl;
	cout << "Total population: " << totalOf(populations) << endl;

	return 0;
}


